#include "morph.h"

#include <QDebug>

Morph::Morph(DataState *state): state(state)
{
}

void Morph::linearMorphing(int fromFrame, int toFrame)
{

    if(toFrame==-1)toFrame=state->vid->frameCount-1;

    QList<QPair<int, int> > framePairs = getFramePairs(fromFrame, toFrame);

    QPair<int, int> pair;
    foreach(pair,framePairs){
        if(pair.first!=-1 && pair.second!=-1){
            int iterator=pair.first;

            QList<QPolygonF> morphedPolygons=getMorphedPolygons(state->getCurrentObject()->keyFrames.at(pair.first)->polygon,state->getCurrentObject()->keyFrames.at(pair.second)->polygon,pair.second-pair.first-1);
            foreach(QPolygonF polygon,morphedPolygons){
                iterator++;
                state->getCurrentObject()->keyFrames.at(iterator)->setNewPolygon(polygon);
                state->getCurrentObject()->keyFrames.at(iterator)->setType(KeyFrame::Automatic);
            }
        }
    }
}

QList<QPair<int, int> > Morph::getFramePairs(int fromFrame, int toFrame)
{

    QList<FrameSpan*> spans = state->getCurrentObject()->getFrameSpans();
    QList<QPair<int, int> > framePairs;

    foreach (FrameSpan* span, spans) {
        if(span->type!=FrameSpan::NotSet && span->prevFrame>=fromFrame && span->nextFrame<=toFrame)
            framePairs.append(QPair<int,int>(span->prevFrame,span->nextFrame));
    }

    QPair<int, int> pair;
    foreach (pair, framePairs) {
        if(state->getCurrentObject()->keyFrames.at(pair.first)->polygon.isEmpty() && !state->getCurrentObject()->keyFrames.at(pair.second)->polygon.isEmpty()){
            state->getCurrentObject()->keyFrames.at(pair.first)->setNewPolygon(state->getCurrentObject()->keyFrames.at(pair.second)->polygon);
            state->getCurrentObject()->keyFrames.at(pair.first)->setType(KeyFrame::Automatic);
        }
        else if(!state->getCurrentObject()->keyFrames.at(pair.first)->polygon.isEmpty() && state->getCurrentObject()->keyFrames.at(pair.second)->polygon.isEmpty()){
            state->getCurrentObject()->keyFrames.at(pair.second)->setNewPolygon(state->getCurrentObject()->keyFrames.at(pair.first)->polygon);
            state->getCurrentObject()->keyFrames.at(pair.second)->setType(KeyFrame::Automatic);
        }
        else if(state->getCurrentObject()->keyFrames.at(pair.first)->polygon.isEmpty() && state->getCurrentObject()->keyFrames.at(pair.second)->polygon.isEmpty()){
            pair.first=-1;
            pair.second=-1;
        }
    }

    if(framePairs.empty())
        framePairs.append(QPair<int,int>(-1,-1));
    return framePairs;
}

QList<QPolygonF> Morph::getMorphedPolygons(QPolygonF left, QPolygonF right, int steps)
{
    QPointF leftC = left.boundingRect().center();
    QPointF rightC = right.boundingRect().center();
    QPointF translate = rightC - leftC;


    // translate and copy polygones
    QPolygonF lPoly = left.translated(translate);
    QPolygonF rPoly = right;


    // check both for counter-clockwise orientation
    int sum = 0;
    for(int i=0; i<lPoly.length(); i++) {
        QPointF p1 = lPoly[i];
        QPointF p2 = lPoly[(i+1)%lPoly.length()];
        sum += (p2.x() - p1.x()) * (p2.y() + p1.y());
    }
    if(sum < 0) {
        QPolygonF tmp;
        for(int i = lPoly.length()-1; i >= 0; i--) {
            tmp << lPoly[i];
        }
        lPoly.swap(tmp);
    }
    sum = 0;
    for(int i=0; i<rPoly.length(); i++) {
        QPointF p1 = rPoly[i];
        QPointF p2 = rPoly[(i+1)%rPoly.length()];
        sum += (p2.x() - p1.x()) * (p2.y() + p1.y());
    }
    if(sum < 0) {
        QPolygonF tmp;
        for(int i = rPoly.length()-1; i >= 0; i--) {
            tmp << rPoly[i];
        }
        rPoly.swap(tmp);
    }


    QMap<qreal,qreal> leftAssignments = assign(lPoly, rPoly);
    QMap<qreal,qreal> rightAssignments = assign(rPoly, lPoly);

    // TODO: pre filtering assignment?

    foreach(qreal key, leftAssignments.keys()) {
        if(qFloor(leftAssignments[key]) == leftAssignments[key]) {
            rightAssignments.remove(leftAssignments[key]);
        }
    }
    foreach(qreal key, rightAssignments.keys()) {
        if(qFloor(rightAssignments[key]) == rightAssignments[key]) {
            leftAssignments.remove(rightAssignments[key]);
        }
    }

    QList<qreal> aKeys = leftAssignments.keys();
    QList<qreal> aVals = leftAssignments.values();
    QList<qreal> bKeys = rightAssignments.keys();
    QList<qreal> bVals = rightAssignments.values();

    int aValMinIdx = std::min_element(aVals.begin(), aVals.end()) - aVals.begin();
    int bValMinIdx = std::min_element(bVals.begin(), bVals.end()) - bVals.begin();

    qreal lastVal;
    if(aVals.length() > 0) {
        lastVal = aVals[aValMinIdx];
        for(int i=1; i<aVals.count(); i++){
            if(aVals[(i+aValMinIdx) % aVals.count()] < lastVal) {
                aVals[(i+aValMinIdx) % aVals.count()] = lastVal;
            }
            lastVal = aVals[(i+aValMinIdx) % aVals.count()];
        }
    }
    if(bVals.length() > 0) {
        lastVal = bVals[bValMinIdx];
        for(int i=1; i<bVals.count(); i++){
            if(bVals[(i+bValMinIdx) % bVals.count()] < lastVal) {
                bVals[(i+bValMinIdx) % bVals.count()] = lastVal;
            }
            lastVal = bVals[(i+bValMinIdx) % bVals.count()];
        }
    }


    // shift lists to have minimum element up front
    int aKeyMinIdx = std::min_element(aKeys.begin(), aKeys.end()) - aKeys.begin();
    for(int i=0; i<aKeyMinIdx; i++) {
        aKeys.append(aKeys.takeFirst());
        aVals.append(aVals.takeFirst());
    }
    for(int i=0; i<bValMinIdx; i++) {
        bKeys.append(bKeys.takeFirst());
        bVals.append(bVals.takeFirst());
    }



    QPolygonF aPoly;
    QPolygonF bPoly;

    while(!aKeys.empty() || !bKeys.empty()) {
        if(bKeys.empty() || (!aKeys.empty() && aKeys.first() < bVals.first())) {
            aPoly << pointFromAssignment(lPoly, aKeys.first());
            bPoly << pointFromAssignment(rPoly, aVals.first());
            aKeys.removeFirst();
            aVals.removeFirst();
        } else {
            aPoly << pointFromAssignment(lPoly, bVals.first());
            bPoly << pointFromAssignment(rPoly, bKeys.first());
            bKeys.removeFirst();
            bVals.removeFirst();
        }
    }
    // translate back
    aPoly.translate((-1)*translate);

    // interpolation
    QList<QPolygonF> list;
    for(qreal s=1; s<=steps; ++s) {
        QPolygonF tmp;
        for(int i=0; i<aPoly.size(); ++i) {
            tmp << (aPoly[i] + (s/(steps+1))*(bPoly[i]-aPoly[i]));
        }
        list << tmp;
    }
    return list;

}

QMap<qreal, qreal> Morph::assign(QPolygonF left, QPolygonF right)
{
    QMap<qreal,qreal> assignments;
    // loop over all points of left polygon
    for(int idx=0; idx < left.count(); idx++) {
        QPointF v = left[idx];
        // compare to all edges of right polygon
        qreal minDist = std::numeric_limits<qreal>::max();
        qreal minAssignment;
        for(int otherIdx = 0; otherIdx < right.count(); otherIdx++) {
            // edge vector
            QPointF a = right[otherIdx];
            QPointF b = right[(otherIdx + 1) % right.count()];
            QPointF e = b - a;
            QPointF e_n = 1.0/qSqrt(QPointF::dotProduct(e,e)) * e;
            // orthogonal projection and interpolation
            QPointF av = v - a;
            qreal t = QPointF::dotProduct(av,e_n) / qSqrt(QPointF::dotProduct(e,e));
            QPointF vproj;
            if(t <= 0) {
                // point is closest to a
                t = 0;
            } else if(t >= 1) {
                // point is closest to b
                t = 1;
            }

            // get projected point
            vproj = a + t*e;

            qreal dist = qSqrt(QPointF::dotProduct(v-vproj, v-vproj));
            if(dist < minDist) {
                minDist = dist;
                minAssignment = otherIdx + t;
            }
        }
        assignments[idx] = minAssignment;
    }
    return assignments;
}

QPointF Morph::pointFromAssignment(QPolygonF poly, qreal assign)
{
    int idx = qFloor(assign);
    qreal t = assign - idx;

    QPointF a = poly[idx % poly.count()];
    QPointF b = poly[(idx + 1) % poly.count()];
    QPointF e = b - a;

    QPointF point = a + t*e;
    return point;
}
