#include "framenav.h"

#include <QDebug>
#include <QGridLayout>
#include <QSizePolicy>

#include "../globalobjects.h"


FrameNav::FrameNav(QWidget *parent, Qt::WindowFlags flags) : FrameNav("", parent, flags) {};

FrameNav::FrameNav(const QString &title, QWidget *parent, Qt::WindowFlags flags) :
    QDockWidget(title, parent, flags)
{
    QWidget* w = new QWidget(this);
    if(w->layout()) delete w->layout();
    QGridLayout* l = new QGridLayout();
    w->setLayout(l);

    buttonFirst = new QPushButton();
    //buttonFirst->setText("|<");
    buttonFirst->setIcon(QIcon(":icons/goStart.ico"));
    l->addWidget(buttonFirst, 0, 0);
    connect(buttonFirst, SIGNAL(clicked()), this, SLOT(onButtonFirstClicked()));

    buttonPrev = new QPushButton();
    //buttonPrev->setText("<");
    buttonPrev->setIcon(QIcon(":icons/goPre.ico"));
    l->addWidget(buttonPrev, 0, 1);
    connect(buttonPrev, SIGNAL(clicked()), this, SLOT(onButtonPrevClicked()));

    spinBox = new QSpinBox();
    spinBox->setButtonSymbols(QAbstractSpinBox::NoButtons);
    spinBox->setMinimum(1);
    spinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    l->addWidget(spinBox, 0, 2);
    connect(spinBox, SIGNAL(editingFinished()), this, SLOT(onSpinBoxEditingFinished()));


    buttonNext = new QPushButton();
    //buttonNext->setText(">");
    buttonNext->setIcon(QIcon(":icons/goNext.ico"));
    l->addWidget(buttonNext, 0, 3);
    connect(buttonNext, SIGNAL(clicked()), this, SLOT(onButtonNextClicked()));

    buttonLast = new QPushButton();
    //buttonLast->setText(">|");
    buttonLast->setIcon(QIcon(":icons/goEnd.ico"));
    l->addWidget(buttonLast, 0, 4);
    connect(buttonLast, SIGNAL(clicked()), this, SLOT(onButtonLastClicked()));

    buttonPlay = new QPushButton();
    buttonPlay->setIcon(QIcon(":icons/play.ico"));
    buttonPlay->setShortcut(Qt::Key_Space);
    l->addWidget(buttonPlay, 0, 5);
    connect(buttonPlay, SIGNAL(clicked()), this, SLOT(onButtonPlayClicked()));

    slider = new QSlider(Qt::Horizontal);
    l->addWidget(slider, 1, 0, 1, 6);
    connect(slider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));


    setWidget(w);

    setDisabled(true);

    connectToState();
}


void FrameNav::setButtonPlaying()
{
    buttonFirst->setEnabled(false);
    buttonLast->setEnabled(false);
    buttonNext->setEnabled(false);
    buttonPrev->setEnabled(false);
    slider->setEnabled(false);
}

void FrameNav::setButtonAllActive()
{
    buttonFirst->setEnabled(true);
    buttonLast->setEnabled(true);
    buttonNext->setEnabled(true);
    buttonPrev->setEnabled(true);
    buttonPlay->setEnabled(true);
    slider->setEnabled(true);
}

void FrameNav::onButtonFirstClicked()
{
    g_state->setFrame(0);
}

void FrameNav::onButtonPrevClicked()
{
    g_state->prevFrame();
}

void FrameNav::onButtonNextClicked()
{
    g_state->nextFrame();
}

void FrameNav::onButtonLastClicked()
{
    int frameCount = g_state->vid->frameCount;
    g_state->setFrame(frameCount-1);
}

void FrameNav::onButtonPlayClicked()
{
    int sFrame=g_state->currentFrame;
    int msPerFrame=round(1000/g_state->vid->fps);
    int i=sFrame;
    int currentFrameInMs=sFrame*msPerFrame;

    QTime playingTime;
    stopPlay=!stopPlay;
    if(stopPlay==false){
        buttonPlay->setIcon(QIcon(":icons/pause.ico"));
        setButtonPlaying();
        playingTime.start();
        int iTemp=-1;
        while(i<g_state->vid->frameCount && !stopPlay){
            i=round((currentFrameInMs+playingTime.elapsed())/msPerFrame);
            if(i!=iTemp){
                g_state->setFrame(i);
                g_mainWindow->frameViewer->setDisabled(true);
                qApp->processEvents();
                iTemp=i;
            }
            if(i>=g_state->vid->frameCount){
                g_state->setFrame(g_state->vid->frameCount-1);
            }
        }
    }
    stopPlay=true;
    buttonPlay->setIcon(QIcon(":icons/play.ico"));
    g_mainWindow->frameViewer->setDisabled(false);
    setButtonAllActive();
}

void FrameNav::onSpinBoxEditingFinished()
{
    int v = spinBox->value();
    g_state->setFrame(v-1);
}

void FrameNav::onSliderValueChanged(int v)
{
    g_state->setFrame(v-1);
}

void FrameNav::connectToState()
{
    this->disconnect(g_state);
    connect(g_state, SIGNAL(hasChanged(int)), this, SLOT(filterStateChange(int)));
}

void FrameNav::filterStateChange(int f)
{
    if(f & DataState::CurrentFrameChange) reload();
}

void FrameNav::reload()
{
    if(!g_state->videoLoaded) {
        qDebug() << "no vid set";
        setDisabled(true);
        return;
    }
    setDisabled(false);

    int frameCount = g_state->vid->frameCount;
    int currentFrame = g_state->currentFrame;

    spinBox->setValue(currentFrame+1);
    spinBox->setMaximum(frameCount);


    slider->setMinimum(1);
    slider->setMaximum(frameCount);
    slider->setValue(currentFrame+1);
}



