#ifndef VIDEOCONTAINER_H
#define VIDEOCONTAINER_H

#include <QObject>
#include <QtCore>
#include <QTemporaryDir>
#include <QTemporaryFile>
#include <QDataStream>

class VideoContainer : public QObject
{
    Q_OBJECT

public:
    explicit VideoContainer(QObject *parent = 0);

    //define << operator
    friend QDataStream& operator<<(QDataStream& out, const VideoContainer &outPut){
        //out<<outPut.tmpDir;//do not copy tmpDir
        //out<<outPut.videoFilePath;//do not copy videoFilePath
        out<<outPut.fps;
        out<<outPut.frameCount;
        out<<outPut.duration;
        out<<outPut.vidSize;
        //out<<outPut.frameVector;//do not copy frameVector
        return out;
    }
    //define >> operator
    friend QDataStream& operator>>(QDataStream& in, VideoContainer &inPut){
        //in>>inPut.tmpDir;//do not copy tmpDir
        //in>>inPut.videoFilePath;//do not copy videoFilePath
        in>>inPut.fps;
        in>>inPut.frameCount;
        in>>inPut.duration;
        in>>inPut.vidSize;
        //in>>outPut.frameVector;//do not copy frameVector;
         return in;
    }

    bool loadFile(QString fileName);
    QString getFramePath(int frame);

    QTemporaryDir tmpDir;
    QString videoFilePath;

    double fps;
    int frameCount;
    double duration;
    QSize vidSize;
    QList<int> frameVector;

private slots:
    void setCancel();

private:

    struct videoToFramesWrapper {
      VideoContainer *instance;
      // cppcheck-suppress noExplicitConstructor
      videoToFramesWrapper(VideoContainer *w): instance(w) {}
      void operator()(int &iteration) {
        instance->videoToFrames(iteration);
      }
    };
    void videoToFrames(int &iteration);

    volatile bool abortVideoToFrames = false;



};

#endif // VIDEOCONTAINER_H
