#include "timeline.h"

#include <QGridLayout>
#include <QGraphicsTextItem>
#include <QScrollBar>
#include <QPen>
#include <QVBoxLayout>
#include <QLabel>
#include <QLayoutItem>

#include "trackedobjectlabel.h"
#include "../globalobjects.h"


void TimeLine::Init()
{

    QWidget* w = new QWidget(this);
    if(w->layout()) delete w->layout();
    QGridLayout* l = new QGridLayout(0);
    w->setLayout(l);

    l->setSpacing(0);
    l->setColumnStretch(1,1);


    addObjectButton = new QPushButton();
    addObjectButton->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Ignored);
    addObjectButton->setText("ADD Object");
    addObjectButton->setIcon(QIcon(":icons/add.ico"));
    addObjectButton->setIconSize(QSize(20,20));
    l->addWidget(addObjectButton, 0,0,1,2);



    scrollArea = new QScrollArea();
    scrollArea->setWidget(new QWidget());
    scrollArea->setWidgetResizable(true);
    scrollArea->setFixedWidth(200);
    scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    QVBoxLayout* vbl = new QVBoxLayout(0);
    scrollArea->widget()->setLayout(vbl);
    vbl->setSpacing(0);
    vbl->setAlignment(Qt::AlignTop);
    vbl->setContentsMargins(0,0,0,0);

    l->addWidget(scrollArea, 1,1,1,1);
    l->addWidget(scrollArea->verticalScrollBar(), 1, 0, 1, 1);



    headerView = new ClickableGraphicsView(this);
    headerView->setFixedHeight(30);
    headerView->setFrameStyle(QFrame::NoFrame);
    headerView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    headerView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    l->addWidget(headerView, 0,2,1,1);

    headerScene = new QGraphicsScene(this);
    headerView->setScene(headerScene);
    headerView->setAlignment(Qt::AlignTop | Qt::AlignLeft);
    headerView->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);


    mainView = new ClickableGraphicsView(this);
    mainView->setFrameStyle(QFrame::NoFrame);
    mainView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    mainView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    l->addWidget(mainView, 1,2,1,1);

    mainScene = new QGraphicsScene(this);
    mainView->setScene(mainScene);
    mainView->setAlignment(Qt::AlignTop | Qt::AlignLeft);
    mainView->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);

    l->addWidget(mainView->horizontalScrollBar(), 2,2,1,1);


    headerFrameMarker = 0;
    mainFrameMarker = 0;

    connect(headerView->horizontalScrollBar(), SIGNAL(valueChanged(int)), mainView->horizontalScrollBar(), SLOT(setValue(int)));
    connect(mainView->horizontalScrollBar(), SIGNAL(valueChanged(int)), headerView->horizontalScrollBar(), SLOT(setValue(int)));

    connect(scrollArea->verticalScrollBar(), SIGNAL(valueChanged(int)), mainView->verticalScrollBar(), SLOT(setValue(int)));
    connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), scrollArea->verticalScrollBar(), SLOT(setValue(int)));

    connect(addObjectButton, SIGNAL(clicked()), this, SLOT(addObject()));


    connect(headerView, SIGNAL(clicked(QPoint)), this, SLOT(viewClicked(QPoint)));
    connect(mainView, SIGNAL(clicked(QPoint)), this, SLOT(viewClicked(QPoint)));


    setWidget(w);

    columnWidth = 40;
    rowHeight = 40;




    connectToState();

    setDisabled(true);
}

void TimeLine::reload()
{
    qDebug() << "TIMELINE RELOAD";
    if(!g_state->videoLoaded) {
        qDebug() << "no vid set";
        setDisabled(true);
        return;
    }
    setDisabled(false);

    reloadList();


    int fc = g_state->vid->frameCount;

    headerScene->clear();
    headerScene->addLine(0,0,fc*columnWidth,0,QPen(QColor(0,0,0,0)));
    for(int i=0; i<fc; ++i) {
        QGraphicsTextItem* txt = headerScene->addText(QString::number(i+1));
        int x = (i+0.5)*columnWidth - 0.5*(txt->boundingRect().width());
        int y = 0;
        txt->setPos(x,y);
    }


    // clear scene
    // remove QObject items first with delete later since they are in the event loop
    foreach(KeyFrameGraphicsItem* item, keyFrameItems) {
        mainScene->removeItem(item);
        item->deleteLater();
    }
    keyFrameItems.clear();
    foreach(FrameSpanGraphicsItem* item, frameSpanItems) {
        mainScene->removeItem(item);
        item->deleteLater();
    }
    frameSpanItems.clear();
    mainScene->clear();

    mainScene->addLine(0,0,fc*columnWidth,0,QPen(QColor(0,0,0,0)));
    if(g_state->hasObjects()) {
        for(int i=0; i<fc; ++i) {
            int x = (i+0.5)*columnWidth;
            int y1 = 0;
            int y2 = g_state->objectsList.length() * rowHeight -1;
            mainScene->addLine(x,y1,x,y2);
        }
    }

    headerFrameMarker = headerScene->addRect(0,0,columnWidth, 30, Qt::NoPen, QBrush(QColor(255,127,127)));
    headerFrameMarker->setZValue(-1);
    mainFrameMarker = mainScene->addRect(0,0,columnWidth, rowHeight * g_state->objectsList.length(), Qt::NoPen, QBrush(QColor(255,127,127)));
    mainFrameMarker->setZValue(-1);

    reloadMarkers();


    QList<TrackedObject *> sortedObjectsList = g_state->getZOrderSortedList(0);
    for(int obj_i=0; obj_i<sortedObjectsList.length(); ++obj_i) {
        TrackedObject * obj = sortedObjectsList[obj_i];
        int i=0;
        while(i < fc-1) {
            FrameSpan *span = obj->frameSpans[i];
            int x1 = (i+0.5)*columnWidth;
            int x2 = (span->nextFrame + 0.5)*columnWidth;

            int y1 = (obj->getZOrderRank(i)+0.5)*rowHeight;
            int y2 = (obj->getZOrderRank(span->nextFrame)+0.5)*rowHeight;
            frameSpanItems.append(new FrameSpanGraphicsItem(mainScene, QPointF(x1,y1), QPointF(x2,y2), span));
            i = span->nextFrame;
        }
    }

    for(int obj_i=0; obj_i<sortedObjectsList.length(); ++obj_i) {
        TrackedObject * obj = sortedObjectsList[obj_i];
        for(int i=0; i<fc; ++i) {
            int y = (obj->getZOrderRank(i)+0.5)*rowHeight;
            int x = (i+0.5)*columnWidth;
            keyFrameItems.append(new KeyFrameGraphicsItem(mainScene, QPointF(x,y), obj->keyFrames[i]));
        }
    }


}

void TimeLine::reloadList()
{
    while (QLayoutItem* i = scrollArea->widget()->layout()->takeAt(0)) {
        i->widget()->deleteLater();
        delete i;
    }
    foreach(TrackedObject* obj, g_state->getZOrderSortedList(0)) {
        TrackedObjectLabel* w = new TrackedObjectLabel(obj->id);
        scrollArea->widget()->layout()->addWidget(w);
        qDebug() << "added object" << obj->name;
    }
}

void TimeLine::reloadMarkers()
{
    if(!headerFrameMarker || !mainFrameMarker) return;
    int x = (g_state->currentFrame)*columnWidth;
    headerFrameMarker->setPos(x,0);
    mainFrameMarker->setPos(x,0);

    mainView->ensureVisible(mainFrameMarker);
}

void TimeLine::addObject()
{    
    if(!g_state->hasObjects()){
        g_state->addObject();
        g_mainWindow->reload();
    }
    else{
        g_state->addObject();
    }

}

void TimeLine::connectToState()
{
    this->disconnect(g_state);
    connect(g_state, SIGNAL(hasChanged(int)), this, SLOT(filterStateChange(int)));
    connectToObjects();
}

void TimeLine::filterStateChange(int f)
{
    if(f & DataState::ObjectsListChange) {
        connectToObjects();
        reload();
    }
//    if(f & (DataState::CurrentObjectChange | DataState::ObjectsListChange)) reload();
    if(f & DataState::CurrentFrameChange) reloadMarkers();
}

void TimeLine::connectToObjects()
{
    if(!g_state->hasObjects()) return;

    foreach(QMetaObject::Connection c, objectConnections) disconnect(c);
    objectConnections.clear();

    foreach(TrackedObject* obj, g_state->objectsList) {
        objectConnections.append(connect(obj, SIGNAL(hasChanged(int)), this, SLOT(filterObjectsChange(int))));
    }

    qDebug() << "connected to all objects";
}

void TimeLine::filterObjectsChange(int f)
{
    qDebug() << "timeline got object change"<<f;
    reload();
}

void TimeLine::viewClicked(QPoint pos)
{
    int scrollPos = pos.x() + mainView->horizontalScrollBar()->value();
    int frameClicked = scrollPosToFrame(scrollPos);
    g_state->setFrame(frameClicked);
}

int TimeLine::scrollPosToFrame(int pos)
{
    return qFloor(((qreal)pos)/columnWidth);
}
